/* $LAAS: api.c 2008/07/15 11:01:18 mallet $ */

/*
 * Copyright (c) 2008 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 *
 *					Anthony Mallet on Fri Jul  4 2008
 */

#include <err.h>
#include <errno.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>

#include "evart-client.h"


/* --- local data ---------------------------------------------------- */

static unsigned short sport =		EVAS_STREAM_PORT;
static unsigned char shost[1024] =	EVAS_STREAM_HOST;
struct sockaddr_in saddr;
static int ssockfd =			-1;


/* --- evas_setport -------------------------------------------------- */

void
evas_setport(unsigned short port)
{
  sport = port;

  if (ssockfd < 0) {
    ssockfd = -1;
    close(ssockfd);
  }
}


/* --- evas_sethost -------------------------------------------------- */

void
evas_sethost(const char *host)
{
  strncpy(shost, host?host:"localhost", sizeof(shost)-1);

  if (ssockfd < 0) {
    ssockfd = -1;
    close(ssockfd);
  }
}


/* --- evas_text ----------------------------------------------------- */

int
evas_text(const char *text)
{
  evas_msg_t msg = { .type = EVAS_MSG };
  int n;

  n = snprintf(msg.cmd, sizeof(msg.cmd), "%s", text);
  msg.len = (char *)&msg.cmd[n+1] - (char *)&msg;

  return evas_send(&msg);
}


/* --- evas_bodymarkers ---------------------------------------------- */

int
evas_bodymarkers(const char *body, evas_onoff_t on)
{
  evas_msg_t msg = { .type = EVAS_BODY_MARKERS };
  int n;

  n = snprintf(msg.cmd, sizeof(msg.cmd), "%c%s", on?'+':'-', body);
  msg.len = (char *)&msg.cmd[n+1] - (char *)&msg;

  return evas_send(&msg);
}


/* --- evas_unrecognized_markers ------------------------------------- */

int
evas_unrecognized_markers(evas_onoff_t on)
{
  evas_msg_t msg = { .type = EVAS_UNRECOGNIZED_MARKERS };
  int n;

  n = snprintf(msg.cmd, sizeof(msg.cmd), "%c", on?'+':'-');
  msg.len = (char *)&msg.cmd[n+1] - (char *)&msg;

  return evas_send(&msg);
}


/* --- evas_send ----------------------------------------------------- */

int
evas_send(const evas_msg_t *msg)
{
  ssize_t sz;

  if (ssockfd < 0) {
    char hport[6];
    struct addrinfo *haddr, *a;
    int reuse;
    int s;

    ssockfd = socket(AF_INET, SOCK_DGRAM, 0);
    if (ssockfd == -1) return -1;

    /* set reuse addr option */
    reuse = 1;
    if (setsockopt(ssockfd, SOL_SOCKET, SO_REUSEADDR,
		   (char *)&reuse, sizeof(reuse))) {
      warnx("cannot set reuseaddr on server socket");
      close(ssockfd);
      ssockfd = -1;
      return -1;
    }

    /* resolve host address */
    sprintf(hport, "%d", sport);
    s = getaddrinfo(shost, hport, NULL, &haddr);
    if (s) { warnx("cannot get remote host addresses"); return s; }

    saddr.sin_addr.s_addr = htonl(INADDR_ANY);
    for(a = haddr; a; a = a->ai_next) switch (a->ai_family) {
      case AF_INET:
	saddr = *(struct sockaddr_in *)a->ai_addr;
	break;

      default: break;
    }
    if (saddr.sin_addr.s_addr == htonl(INADDR_ANY)) {
      warnx("cannot resolve remote host address");
      close(ssockfd);
      ssockfd = -1;
      return -1;
    }
  }

  sz = sendto(ssockfd, (const char *)msg, msg->len, 0,
	      (struct sockaddr *)&saddr, sizeof(saddr));
  if (sz != msg->len) return -1;

  return 0;
}


/* --- evas_recv ----------------------------------------------------- */

const evas_msg_t *
evas_recv()
{
  static char buffer[65536];
  evas_msg_t *msg = (evas_msg_t *)buffer;
  struct sockaddr from;
  int fromlen;
  int s;

  if (ssockfd < 0) return NULL;

  do {
    s = recvfrom(ssockfd, buffer, sizeof(buffer), 0, &from, &fromlen);
  } while(s == -1 && errno == EAGAIN);

  if (s <= 0) return NULL;

  return msg;
}
