/* $LAAS: evart-client.h 2008/07/15 11:01:06 mallet $ */

/* 
 * Copyright (c) 2008 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 *
 *					Anthony Mallet on Fri Jul  4 2008
 */

#ifndef H_EVART_CLIENT
#define H_EVART_CLIENT

#define EVAS_EMPTY 9999999.0f

/** on/off flag */
typedef enum evas_onoff_t {
  EVAS_OFF,
  EVAS_ON
} evas_onoff_t;

/** the marker positions for one body */
typedef struct evas_body_markers_t {
  char name[128];	/** for dynamic matching of objects. */

  int nmarkers;		/** number of markers defined */
  float markers[0][3];	/** [nmarkers][3] array.
			    markers[i][0] == EVAS_EMPTY means no data. */
} evas_body_markers_t;


/** the unrecognized markers positions */
typedef struct evas_unrecognized_markers_t {
  int nmarkers;		/** number of markers defined */
  float markers[0][3];	/** [nmarkers][3] array.
			    markers[i][0] == EVAS_EMPTY means no data. */
} evas_unrecognized_markers_t;


/* control messages type */
typedef enum evas_msgtype_t {
  EVAS_NONE,
  EVAS_MSG,
  EVAS_BODY_MARKERS,
  EVAS_UNRECOGNIZED_MARKERS
} evas_msgtype_t;

/* control message */
typedef struct evas_msg_t {
  unsigned int len;
  evas_msgtype_t type;
  union {
    char cmd[128];
    evas_body_markers_t body_markers;
    evas_unrecognized_markers_t unrecognized_markers;
  };
} evas_msg_t;

#define EVAS_STREAM_PORT	3615
#define EVAS_STREAM_HOST	"shrek"

/* --- API ----------------------------------------------------------- */

void			evas_setport(unsigned short port);
void			evas_sethost(const char *host);

int			evas_text(const char *text);
int			evas_bodymarkers(const char *body, evas_onoff_t on);
int			evas_unrecognized_markers(evas_onoff_t on);

const evas_msg_t *	evas_recv(void);

#endif /* H_EVART_CLIENT */
